import os.path
from queue import Queue
from defusedxml import sax as defusedxml_sax
from xml.sax.handler import ContentHandler

from parosm.parse.parsebase import BaseParser
from parosm.types import OSM, Node, Way, Relation


class Element:
    """
    XML element
    not an osm object!!!
    """
    def __init__(self, tag, attrib=None):
        self.tag = tag
        self.attrib = dict() if attrib is None else attrib


class OSMContentHandler(ContentHandler):
    """
    Content Handler for XML data
    """
    def __init__(self):
        """
        Initialize content handler
        """
        super().__init__()
        self._event_queue = Queue()

    def read_events(self):
        """
        Read the read xml events
        :return: xml parser event
        """
        while not self._event_queue.empty():
            yield self._event_queue.get()

    def startElement(self, name, attrs):
        """
        Callback when an element starts
        Signals the start of an element in non-namespace mode.
        :param name: raw xml name
        :param attrs: Attributes interface object
        """
        attrib = {key: attrs.get(key) for key in attrs.getNames()}
        self._event_queue.put(('start', Element(name, attrib)))

    def endElement(self, name):
        """
        Signals the end of an element in non-namespace mode.
        :param name: raw xml name
        """
        self._event_queue.put(('end', Element(name)))


class XMLParser(BaseParser):
    """
    XMLParser parses the the osm-xml format
    """
    def __init__(self, file, callback=None):
        """
        Initialize XMLParser

        The callback-method is called when a osm object is found

        def callback(element):
            pass

        :param file: Path to file
        :param callback: Callback for read osm objects
        """
        super().__init__(file, callback)
        self.__file = file
        if not os.path.isfile(file):
            raise Exception('is not a file')

        self.__callback = self.__default_callback \
            if callback is None else callback
        self.__parser = defusedxml_sax.make_parser()
        self.__handler = OSMContentHandler()

        self.__parser.setContentHandler(self.__handler)

        self.__in_node = False
        self.__in_way = False
        self.__in_relation = False
        self.__in_osm = False
        self.__last_event = None

        self.__osm_object = None
        self.__current_object = None

    @staticmethod
    def __default_callback(element):
        """
        Default callback when no callback is given in init method
        :param element: osm object
        """
        print(str(element))

    def parse(self):
        """
        Starts the parser
        """
        with open(self.__file, 'r') as f:
            for ln, line in enumerate(f):
                try:
                    self.__parse_internal(line)
                except AttributeError as e:
                    print(ln)
                    raise e

    def __parse_internal(self, line):
        """
        Internal osm object orchestration from the file
        :param line: current
        """
        self.__parser.feed(line)
        for event, element in self.__handler.read_events():
            if element.tag == 'osm' and event == 'start':
                self.__in_osm = True
                self.__osm_object = OSM(element.attrib['version'])
            elif element.tag == 'osm' and event == 'end':
                self.__in_osm = False
                self.__callback(self.__osm_object)
            elif element.tag == 'tag' and event == 'start':
                key = element.attrib['k']
                value = element.attrib['v']
                self.__current_object.add_tag(key, value)
            elif element.tag == 'bounds' and event == 'start':
                self.__osm_object.set_bounds(**element.attrib)
            elif element.tag == 'bounds' and event == 'end':
                pass
            elif element.tag == 'node' and event == 'start':
                attrs = element.attrib
                self.__current_object = Node(identifier=attrs['id'], **attrs)
                self.__in_node = True
            elif element.tag == 'node' and event == 'end':
                self.__callback(self.__current_object)
                self.__in_node = False
                self.__current_object = None
            elif element.tag == 'way' and event == 'start':
                attrs = element.attrib
                self.__current_object = Way(identifier=attrs['id'], **attrs)
                self.__in_way = True
            elif element.tag == 'way' and event == 'end':
                self.__callback(self.__current_object)
                self.__in_way = False
                self.__current_object = None
            elif element.tag == 'nd' and event == 'start' and self.__in_way:
                self.__current_object.add_node(element.attrib['ref'])
            elif element.tag == 'relation' and event == 'start':
                attrs = element.attrib
                self.__current_object = Relation(identifier=attrs['id'], **attrs)
                self.__in_relation = True
            elif element.tag == 'relation' and event == 'end':
                self.__callback(self.__current_object)
                self.__in_relation = False
                self.__current_object = None
            elif element.tag == 'member' and event == 'start' and self.__in_relation:
                attrs = element.attrib
                self.__current_object.add_member(attrs['ref'],
                                                 attrs['type'],
                                                 attrs['role'])
            elif element.tag == 'member' and event == 'end':
                pass




"""
Exposes parsers for osm files
"""

from parosm.parse.parsexml import XMLParser
from parosm.parse.parsepbf import PBFParser
from parosm.parse.parsemulti import MultiParser


__all__ = ('XMLParser', 'PBFParser', 'MultiParser')



from parosm.types.osmtype import OSMBaseType


class Relation(OSMBaseType):
    def __init__(self, identifier, members=None, **kwargs):
        """

        :param identifier: Object ID
        :param members: Dict[id: Tuple[MemberType, Role]]
        :param user: user name
        :param uid: user id
        :param timestamp: timestamp of last change
        :param visible: object visible?
        :param version: version of the object
        :param changeset: changeset id
        :param tags: dictionary with osm object tags
        """
        super().__init__(identifier, **kwargs)
        self._members = dict() if members is None else members

    def add_member(self, identifier, mtype, role):
        self._members[identifier] = (mtype, role)

    @property
    def members(self):
        for identifier, (mtype, role) in self._members.items():
            yield (identifier, mtype, role)

    def __str__(self):
        attr_str = list()
        attr_str.append('(id={})'.format(self._id))
        attr_str.append('(members={})'.format(len(self._members)))
        if len(self._tags) > 0:
            attr_str.append('(tags={})'.format(self._tags))
        attr_str = ', '.join(attr_str)
        return 'Relation<{}>;'.format(attr_str)



from abc import ABC, abstractmethod


class BaseParser:
    """
    This is the base class for Parsers
    it defines the interface for a parser
    """
    @abstractmethod
    def __init__(self, file, callback=None):
        """
        Initializes the parser

        :param file: path to file
        :param callback: callback is called with element
        """
        pass

    @abstractmethod
    def parse(self):
        """
        Start the parsing process
        """
        pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: osmformat.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='osmformat.proto',
  package='OSMPBF',
  syntax='proto2',
  serialized_pb=_b('\n\x0fosmformat.proto\x12\x06OSMPBF\"\x87\x02\n\x0bHeaderBlock\x12 \n\x04\x62\x62ox\x18\x01 \x01(\x0b\x32\x12.OSMPBF.HeaderBBox\x12\x19\n\x11required_features\x18\x04 \x03(\t\x12\x19\n\x11optional_features\x18\x05 \x03(\t\x12\x16\n\x0ewritingprogram\x18\x10 \x01(\t\x12\x0e\n\x06source\x18\x11 \x01(\t\x12%\n\x1dosmosis_replication_timestamp\x18  \x01(\x03\x12+\n#osmosis_replication_sequence_number\x18! \x01(\x03\x12$\n\x1cosmosis_replication_base_url\x18\" \x01(\t\"F\n\nHeaderBBox\x12\x0c\n\x04left\x18\x01 \x02(\x12\x12\r\n\x05right\x18\x02 \x02(\x12\x12\x0b\n\x03top\x18\x03 \x02(\x12\x12\x0e\n\x06\x62ottom\x18\x04 \x02(\x12\"\xd2\x01\n\x0ePrimitiveBlock\x12(\n\x0bstringtable\x18\x01 \x02(\x0b\x32\x13.OSMPBF.StringTable\x12.\n\x0eprimitivegroup\x18\x02 \x03(\x0b\x32\x16.OSMPBF.PrimitiveGroup\x12\x18\n\x0bgranularity\x18\x11 \x01(\x05:\x03\x31\x30\x30\x12\x15\n\nlat_offset\x18\x13 \x01(\x03:\x01\x30\x12\x15\n\nlon_offset\x18\x14 \x01(\x03:\x01\x30\x12\x1e\n\x10\x64\x61te_granularity\x18\x12 \x01(\x05:\x04\x31\x30\x30\x30\"\xb7\x01\n\x0ePrimitiveGroup\x12\x1b\n\x05nodes\x18\x01 \x03(\x0b\x32\x0c.OSMPBF.Node\x12!\n\x05\x64\x65nse\x18\x02 \x01(\x0b\x32\x12.OSMPBF.DenseNodes\x12\x19\n\x04ways\x18\x03 \x03(\x0b\x32\x0b.OSMPBF.Way\x12#\n\trelations\x18\x04 \x03(\x0b\x32\x10.OSMPBF.Relation\x12%\n\nchangesets\x18\x05 \x03(\x0b\x32\x11.OSMPBF.ChangeSet\"\x18\n\x0bStringTable\x12\t\n\x01s\x18\x01 \x03(\x0c\"q\n\x04Info\x12\x13\n\x07version\x18\x01 \x01(\x05:\x02-1\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\x11\n\tchangeset\x18\x03 \x01(\x03\x12\x0b\n\x03uid\x18\x04 \x01(\x05\x12\x10\n\x08user_sid\x18\x05 \x01(\r\x12\x0f\n\x07visible\x18\x06 \x01(\x08\"\x8a\x01\n\tDenseInfo\x12\x13\n\x07version\x18\x01 \x03(\x05\x42\x02\x10\x01\x12\x15\n\ttimestamp\x18\x02 \x03(\x12\x42\x02\x10\x01\x12\x15\n\tchangeset\x18\x03 \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03uid\x18\x04 \x03(\x11\x42\x02\x10\x01\x12\x14\n\x08user_sid\x18\x05 \x03(\x11\x42\x02\x10\x01\x12\x13\n\x07visible\x18\x06 \x03(\x08\x42\x02\x10\x01\"\x17\n\tChangeSet\x12\n\n\x02id\x18\x01 \x02(\x03\"l\n\x04Node\x12\n\n\x02id\x18\x01 \x02(\x12\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x0b\n\x03lat\x18\x08 \x02(\x12\x12\x0b\n\x03lon\x18\t \x02(\x12\"{\n\nDenseNodes\x12\x0e\n\x02id\x18\x01 \x03(\x12\x42\x02\x10\x01\x12$\n\tdenseinfo\x18\x05 \x01(\x0b\x32\x11.OSMPBF.DenseInfo\x12\x0f\n\x03lat\x18\x08 \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03lon\x18\t \x03(\x12\x42\x02\x10\x01\x12\x15\n\tkeys_vals\x18\n \x03(\x05\x42\x02\x10\x01\"c\n\x03Way\x12\n\n\x02id\x18\x01 \x02(\x03\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x10\n\x04refs\x18\x08 \x03(\x12\x42\x02\x10\x01\"\xe0\x01\n\x08Relation\x12\n\n\x02id\x18\x01 \x02(\x03\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x15\n\troles_sid\x18\x08 \x03(\x05\x42\x02\x10\x01\x12\x12\n\x06memids\x18\t \x03(\x12\x42\x02\x10\x01\x12.\n\x05types\x18\n \x03(\x0e\x32\x1b.OSMPBF.Relation.MemberTypeB\x02\x10\x01\"-\n\nMemberType\x12\x08\n\x04NODE\x10\x00\x12\x07\n\x03WAY\x10\x01\x12\x0c\n\x08RELATION\x10\x02\x42\x11\n\rcrosby.binaryH\x03')
)



_RELATION_MEMBERTYPE = _descriptor.EnumDescriptor(
  name='MemberType',
  full_name='OSMPBF.Relation.MemberType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NODE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAY', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELATION', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1587,
  serialized_end=1632,
)
_sym_db.RegisterEnumDescriptor(_RELATION_MEMBERTYPE)


_HEADERBLOCK = _descriptor.Descriptor(
  name='HeaderBlock',
  full_name='OSMPBF.HeaderBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bbox', full_name='OSMPBF.HeaderBlock.bbox', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='required_features', full_name='OSMPBF.HeaderBlock.required_features', index=1,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='optional_features', full_name='OSMPBF.HeaderBlock.optional_features', index=2,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='writingprogram', full_name='OSMPBF.HeaderBlock.writingprogram', index=3,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='OSMPBF.HeaderBlock.source', index=4,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='osmosis_replication_timestamp', full_name='OSMPBF.HeaderBlock.osmosis_replication_timestamp', index=5,
      number=32, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='osmosis_replication_sequence_number', full_name='OSMPBF.HeaderBlock.osmosis_replication_sequence_number', index=6,
      number=33, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='osmosis_replication_base_url', full_name='OSMPBF.HeaderBlock.osmosis_replication_base_url', index=7,
      number=34, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=291,
)


_HEADERBBOX = _descriptor.Descriptor(
  name='HeaderBBox',
  full_name='OSMPBF.HeaderBBox',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='left', full_name='OSMPBF.HeaderBBox.left', index=0,
      number=1, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='right', full_name='OSMPBF.HeaderBBox.right', index=1,
      number=2, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='top', full_name='OSMPBF.HeaderBBox.top', index=2,
      number=3, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bottom', full_name='OSMPBF.HeaderBBox.bottom', index=3,
      number=4, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=293,
  serialized_end=363,
)


_PRIMITIVEBLOCK = _descriptor.Descriptor(
  name='PrimitiveBlock',
  full_name='OSMPBF.PrimitiveBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stringtable', full_name='OSMPBF.PrimitiveBlock.stringtable', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='primitivegroup', full_name='OSMPBF.PrimitiveBlock.primitivegroup', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='granularity', full_name='OSMPBF.PrimitiveBlock.granularity', index=2,
      number=17, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=100,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lat_offset', full_name='OSMPBF.PrimitiveBlock.lat_offset', index=3,
      number=19, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lon_offset', full_name='OSMPBF.PrimitiveBlock.lon_offset', index=4,
      number=20, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='date_granularity', full_name='OSMPBF.PrimitiveBlock.date_granularity', index=5,
      number=18, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=1000,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=366,
  serialized_end=576,
)


_PRIMITIVEGROUP = _descriptor.Descriptor(
  name='PrimitiveGroup',
  full_name='OSMPBF.PrimitiveGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes', full_name='OSMPBF.PrimitiveGroup.nodes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dense', full_name='OSMPBF.PrimitiveGroup.dense', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ways', full_name='OSMPBF.PrimitiveGroup.ways', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relations', full_name='OSMPBF.PrimitiveGroup.relations', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='changesets', full_name='OSMPBF.PrimitiveGroup.changesets', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=579,
  serialized_end=762,
)


_STRINGTABLE = _descriptor.Descriptor(
  name='StringTable',
  full_name='OSMPBF.StringTable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='s', full_name='OSMPBF.StringTable.s', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=764,
  serialized_end=788,
)


_INFO = _descriptor.Descriptor(
  name='Info',
  full_name='OSMPBF.Info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='OSMPBF.Info.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='OSMPBF.Info.timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='changeset', full_name='OSMPBF.Info.changeset', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='OSMPBF.Info.uid', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_sid', full_name='OSMPBF.Info.user_sid', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='visible', full_name='OSMPBF.Info.visible', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=790,
  serialized_end=903,
)


_DENSEINFO = _descriptor.Descriptor(
  name='DenseInfo',
  full_name='OSMPBF.DenseInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='OSMPBF.DenseInfo.version', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='OSMPBF.DenseInfo.timestamp', index=1,
      number=2, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='changeset', full_name='OSMPBF.DenseInfo.changeset', index=2,
      number=3, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='OSMPBF.DenseInfo.uid', index=3,
      number=4, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_sid', full_name='OSMPBF.DenseInfo.user_sid', index=4,
      number=5, type=17, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='visible', full_name='OSMPBF.DenseInfo.visible', index=5,
      number=6, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=906,
  serialized_end=1044,
)


_CHANGESET = _descriptor.Descriptor(
  name='ChangeSet',
  full_name='OSMPBF.ChangeSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='OSMPBF.ChangeSet.id', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1046,
  serialized_end=1069,
)


_NODE = _descriptor.Descriptor(
  name='Node',
  full_name='OSMPBF.Node',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='OSMPBF.Node.id', index=0,
      number=1, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys', full_name='OSMPBF.Node.keys', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vals', full_name='OSMPBF.Node.vals', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='OSMPBF.Node.info', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lat', full_name='OSMPBF.Node.lat', index=4,
      number=8, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lon', full_name='OSMPBF.Node.lon', index=5,
      number=9, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1071,
  serialized_end=1179,
)


_DENSENODES = _descriptor.Descriptor(
  name='DenseNodes',
  full_name='OSMPBF.DenseNodes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='OSMPBF.DenseNodes.id', index=0,
      number=1, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='denseinfo', full_name='OSMPBF.DenseNodes.denseinfo', index=1,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lat', full_name='OSMPBF.DenseNodes.lat', index=2,
      number=8, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lon', full_name='OSMPBF.DenseNodes.lon', index=3,
      number=9, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys_vals', full_name='OSMPBF.DenseNodes.keys_vals', index=4,
      number=10, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1181,
  serialized_end=1304,
)


_WAY = _descriptor.Descriptor(
  name='Way',
  full_name='OSMPBF.Way',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='OSMPBF.Way.id', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys', full_name='OSMPBF.Way.keys', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vals', full_name='OSMPBF.Way.vals', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='OSMPBF.Way.info', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='refs', full_name='OSMPBF.Way.refs', index=4,
      number=8, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1306,
  serialized_end=1405,
)


_RELATION = _descriptor.Descriptor(
  name='Relation',
  full_name='OSMPBF.Relation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='OSMPBF.Relation.id', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys', full_name='OSMPBF.Relation.keys', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vals', full_name='OSMPBF.Relation.vals', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='OSMPBF.Relation.info', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roles_sid', full_name='OSMPBF.Relation.roles_sid', index=4,
      number=8, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memids', full_name='OSMPBF.Relation.memids', index=5,
      number=9, type=18, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='types', full_name='OSMPBF.Relation.types', index=6,
      number=10, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RELATION_MEMBERTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1408,
  serialized_end=1632,
)

_HEADERBLOCK.fields_by_name['bbox'].message_type = _HEADERBBOX
_PRIMITIVEBLOCK.fields_by_name['stringtable'].message_type = _STRINGTABLE
_PRIMITIVEBLOCK.fields_by_name['primitivegroup'].message_type = _PRIMITIVEGROUP
_PRIMITIVEGROUP.fields_by_name['nodes'].message_type = _NODE
_PRIMITIVEGROUP.fields_by_name['dense'].message_type = _DENSENODES
_PRIMITIVEGROUP.fields_by_name['ways'].message_type = _WAY
_PRIMITIVEGROUP.fields_by_name['relations'].message_type = _RELATION
_PRIMITIVEGROUP.fields_by_name['changesets'].message_type = _CHANGESET
_NODE.fields_by_name['info'].message_type = _INFO
_DENSENODES.fields_by_name['denseinfo'].message_type = _DENSEINFO
_WAY.fields_by_name['info'].message_type = _INFO
_RELATION.fields_by_name['info'].message_type = _INFO
_RELATION.fields_by_name['types'].enum_type = _RELATION_MEMBERTYPE
_RELATION_MEMBERTYPE.containing_type = _RELATION
DESCRIPTOR.message_types_by_name['HeaderBlock'] = _HEADERBLOCK
DESCRIPTOR.message_types_by_name['HeaderBBox'] = _HEADERBBOX
DESCRIPTOR.message_types_by_name['PrimitiveBlock'] = _PRIMITIVEBLOCK
DESCRIPTOR.message_types_by_name['PrimitiveGroup'] = _PRIMITIVEGROUP
DESCRIPTOR.message_types_by_name['StringTable'] = _STRINGTABLE
DESCRIPTOR.message_types_by_name['Info'] = _INFO
DESCRIPTOR.message_types_by_name['DenseInfo'] = _DENSEINFO
DESCRIPTOR.message_types_by_name['ChangeSet'] = _CHANGESET
DESCRIPTOR.message_types_by_name['Node'] = _NODE
DESCRIPTOR.message_types_by_name['DenseNodes'] = _DENSENODES
DESCRIPTOR.message_types_by_name['Way'] = _WAY
DESCRIPTOR.message_types_by_name['Relation'] = _RELATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HeaderBlock = _reflection.GeneratedProtocolMessageType('HeaderBlock', (_message.Message,), dict(
  DESCRIPTOR = _HEADERBLOCK,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.HeaderBlock)
  ))
_sym_db.RegisterMessage(HeaderBlock)

HeaderBBox = _reflection.GeneratedProtocolMessageType('HeaderBBox', (_message.Message,), dict(
  DESCRIPTOR = _HEADERBBOX,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.HeaderBBox)
  ))
_sym_db.RegisterMessage(HeaderBBox)

PrimitiveBlock = _reflection.GeneratedProtocolMessageType('PrimitiveBlock', (_message.Message,), dict(
  DESCRIPTOR = _PRIMITIVEBLOCK,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.PrimitiveBlock)
  ))
_sym_db.RegisterMessage(PrimitiveBlock)

PrimitiveGroup = _reflection.GeneratedProtocolMessageType('PrimitiveGroup', (_message.Message,), dict(
  DESCRIPTOR = _PRIMITIVEGROUP,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.PrimitiveGroup)
  ))
_sym_db.RegisterMessage(PrimitiveGroup)

StringTable = _reflection.GeneratedProtocolMessageType('StringTable', (_message.Message,), dict(
  DESCRIPTOR = _STRINGTABLE,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.StringTable)
  ))
_sym_db.RegisterMessage(StringTable)

Info = _reflection.GeneratedProtocolMessageType('Info', (_message.Message,), dict(
  DESCRIPTOR = _INFO,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.Info)
  ))
_sym_db.RegisterMessage(Info)

DenseInfo = _reflection.GeneratedProtocolMessageType('DenseInfo', (_message.Message,), dict(
  DESCRIPTOR = _DENSEINFO,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.DenseInfo)
  ))
_sym_db.RegisterMessage(DenseInfo)

ChangeSet = _reflection.GeneratedProtocolMessageType('ChangeSet', (_message.Message,), dict(
  DESCRIPTOR = _CHANGESET,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.ChangeSet)
  ))
_sym_db.RegisterMessage(ChangeSet)

Node = _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), dict(
  DESCRIPTOR = _NODE,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.Node)
  ))
_sym_db.RegisterMessage(Node)

DenseNodes = _reflection.GeneratedProtocolMessageType('DenseNodes', (_message.Message,), dict(
  DESCRIPTOR = _DENSENODES,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.DenseNodes)
  ))
_sym_db.RegisterMessage(DenseNodes)

Way = _reflection.GeneratedProtocolMessageType('Way', (_message.Message,), dict(
  DESCRIPTOR = _WAY,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.Way)
  ))
_sym_db.RegisterMessage(Way)

Relation = _reflection.GeneratedProtocolMessageType('Relation', (_message.Message,), dict(
  DESCRIPTOR = _RELATION,
  __module__ = 'osmformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.Relation)
  ))
_sym_db.RegisterMessage(Relation)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\rcrosby.binaryH\003'))
_DENSEINFO.fields_by_name['version'].has_options = True
_DENSEINFO.fields_by_name['version']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSEINFO.fields_by_name['timestamp'].has_options = True
_DENSEINFO.fields_by_name['timestamp']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSEINFO.fields_by_name['changeset'].has_options = True
_DENSEINFO.fields_by_name['changeset']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSEINFO.fields_by_name['uid'].has_options = True
_DENSEINFO.fields_by_name['uid']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSEINFO.fields_by_name['user_sid'].has_options = True
_DENSEINFO.fields_by_name['user_sid']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSEINFO.fields_by_name['visible'].has_options = True
_DENSEINFO.fields_by_name['visible']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_NODE.fields_by_name['keys'].has_options = True
_NODE.fields_by_name['keys']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_NODE.fields_by_name['vals'].has_options = True
_NODE.fields_by_name['vals']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSENODES.fields_by_name['id'].has_options = True
_DENSENODES.fields_by_name['id']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSENODES.fields_by_name['lat'].has_options = True
_DENSENODES.fields_by_name['lat']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSENODES.fields_by_name['lon'].has_options = True
_DENSENODES.fields_by_name['lon']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_DENSENODES.fields_by_name['keys_vals'].has_options = True
_DENSENODES.fields_by_name['keys_vals']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_WAY.fields_by_name['keys'].has_options = True
_WAY.fields_by_name['keys']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_WAY.fields_by_name['vals'].has_options = True
_WAY.fields_by_name['vals']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_WAY.fields_by_name['refs'].has_options = True
_WAY.fields_by_name['refs']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_RELATION.fields_by_name['keys'].has_options = True
_RELATION.fields_by_name['keys']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_RELATION.fields_by_name['vals'].has_options = True
_RELATION.fields_by_name['vals']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_RELATION.fields_by_name['roles_sid'].has_options = True
_RELATION.fields_by_name['roles_sid']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_RELATION.fields_by_name['memids'].has_options = True
_RELATION.fields_by_name['memids']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
_RELATION.fields_by_name['types'].has_options = True
_RELATION.fields_by_name['types']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\020\001'))
# @@protoc_insertion_point(module_scope)



from parosm.parse import *
from parosm.types import *


__all__ = ('MultiParser',
           'PBFParser',
           'XMLParser',
           'Node',
           'Way',
           'Relation',
           'OSM',)



import argparse
import os

import magic

from parosm.parse import MultiParser
from parosm.types import *


node_counter = 0
way_counter = 0
relation_counter = 0
osm_obj = None


def parse_args():
    parser = argparse.ArgumentParser(
        description='Read a OSM-XML file and give info'
    )
    parser.add_argument('input',
                        type=str,
                        help='OSM file to read')
    # type_group = parser.add_mutually_exclusive_group()
    # type_group.add_argument('-p', '--pbf',
    #                         action='store_true',
    #                         help='Input file is pbf')
    # type_group.add_argument('-x', '--xml',
    #                         action='store_true',
    #                         help='Input file is xml (default)')
    return parser.parse_args()


def counter(obj):
    global node_counter
    global way_counter
    global relation_counter
    global osm_obj

    if isinstance(obj, Node):
        node_counter += 1
    elif isinstance(obj, Way):
        way_counter += 1
    elif isinstance(obj, Relation):
        relation_counter += 1
    elif isinstance(obj, OSM):
        osm_obj = obj


def main():
    args = parse_args()
    parser = MultiParser(args.input, counter)
    parser.parse()
    print('Bounds: N={}, E={}, S={}, W={}'.format(
        osm_obj.bounds['maxlon'],
        osm_obj.bounds['maxlat'],
        osm_obj.bounds['minlon'],
        osm_obj.bounds['minlat'])
    )
    print('Nodes:     {}'.format(node_counter))
    print('Ways:      {}'.format(way_counter))
    print('Relations: {}'.format(relation_counter))


if __name__ == '__main__':
    main()



import os.path
import zlib
import lzma

from parosm.parse.parsebase import BaseParser
from parosm.parse import fileformat_pb2 as pbf_fformat
from parosm.parse import osmformat_pb2 as pbf_oformat
from parosm.types import *


class PBFParser(BaseParser):
    """
    PBFParser parses the osm-pbf format
    """
    def __init__(self, file, callback=None):
        """
        Initialize PBFParser

        The callback-method is called when a osm object is found

        def callback(element):
            pass

        :param file: Path to file
        :param callback: Callback for read osm objects
        """
        super().__init__(file, callback)
        self.__file = file
        if not os.path.isfile(file):
            raise Exception('is not a file')

        self.__callback = self.__default_callback \
            if callback is None else callback

    @staticmethod
    def __default_callback(element):
        """
        Default callback when no callback is given in init method
        :param element: osm object
        """
        print(str(element))

    def parse(self):
        """
        Starts the parser
        """
        with open(self.__file, 'rb') as f:
            while self.__parse_blob(f):
                pass

    def __parse_blob(self, file):
        """
        Internal parsing function
        parses a full blob
        :param file: open file object
        :return: False when EOF, True if not
        """
        blob_header_length_raw = file.read(4)
        if len(blob_header_length_raw) == 0:
            return False
        blob_header_length = int.from_bytes(blob_header_length_raw, "big")
        blob_header_raw = file.read(blob_header_length)
        blob_header = pbf_fformat.BlobHeader()
        blob_header.ParseFromString(blob_header_raw)
        blob_raw = file.read(blob_header.datasize)
        blob = pbf_fformat.Blob()
        blob.ParseFromString(blob_raw)
        max_blob_len = max(len(blob.raw), len(blob.zlib_data), len(blob.lzma_data))

        data = None
        if len(blob.raw) == max_blob_len:
            data = blob.raw
        elif len(blob.zlib_data) == max_blob_len:
            data = zlib.decompress(blob.zlib_data)
        elif len(blob.lzma_data) == max_blob_len:
            dec = lzma.LZMADecompressor()
            data = dec.decompress(blob.lzma_data)

        if blob_header.type == 'OSMHeader':
            header = pbf_oformat.HeaderBlock()
            header.ParseFromString(data)
            osm = OSM(header.source)
            bounds = {
                'minlat': header.bbox.bottom * (10**-9),
                'maxlat': header.bbox.top * (10**-9),
                'minlon': header.bbox.left * (10**-9),
                'maxlon': header.bbox.right * (10**-9)
            }
            osm.set_bounds(**bounds)
            self.__callback(osm)
        elif blob_header.type == 'OSMData':
            odata = pbf_oformat.PrimitiveBlock()
            odata.ParseFromString(data)
            strings = [s.decode('utf-8') for s in odata.stringtable.s]
            for pg in odata.primitivegroup:
                for ID, lat, lon, kv in self.__parse_dense_nodes(pg.dense):
                    lat += odata.lat_offset
                    lon += odata.lon_offset
                    lat *= odata.granularity * (10**-9)
                    lon *= odata.granularity * (10**-9)
                    tags = {strings[x[0]]: strings[x[1]] for x in kv}
                    self.__callback(Node(ID, lat, lon, tags=tags))
                for node in pg.nodes:
                    lat = node.lat + odata.lat_offset
                    lon = node.lon + odata.lon_offset
                    lat *= odata.granularity * (10**-9)
                    lon *= odata.granularity * (10**-9)
                    tags = {strings[k]: strings[v] for k, v in zip(node.keys, node.vals)}

                    self.__callback(Node(node.id, lat, lon, tags=tags))
                for way in pg.ways:
                    tags = {strings[k]: strings[v] for k, v in zip(way.keys, way.vals)}
                    old_ref = None
                    refs = []
                    for ref in way.refs:
                        if old_ref is not None:
                            ref += old_ref
                        old_ref = ref
                        refs.append(ref)
                    self.__callback(Way(way.id, nodes=refs, tags=tags))
                for rel in pg.relations:
                    tags = {strings[k]: strings[v] for k, v in zip(rel.keys, rel.vals)}
                    old_memid = None
                    memids = []
                    mtype = []

                    for t in rel.types:
                        if t == 0:
                            mtype.append('node')
                        elif t == 1:
                            mtype.append('way')
                        elif t == 2:
                            mtype.append('relation')

                    for memid in rel.memids:
                        if old_memid is not None:
                            memid += old_memid
                        old_memid = memid
                        memids.append(memid)
                    members = {mid: (mtype, strings[role]) for mid, mtype, role in zip(memids, mtype, rel.roles_sid)}
                    self.__callback(Relation(rel.id, members=members, tags=tags))
        return True

    @staticmethod
    def __parse_dense_nodes(dense):
        """
        Parse dense node

        yields a tuple of the node ID, latitude, longitude and a
        tuple with key value attribute values

        :param dense: the dense node object
        :return: Tuple[ID, Latitude, Longitude, Tuple[Key, Value]]
        """
        last_id = None
        last_lat = None
        last_lon = None
        kv_temp = [str(x) for x in dense.keys_vals]
        kvs = [x.strip() for x in (' '.join(kv_temp)).split(' 0')]
        for ID, lat, lon, kv in zip(dense.id, dense.lat, dense.lon, kvs):
            if kv != "":
                ks = [int(k) for i, k in enumerate(kv.split(' ')) if (i % 2) == 0]
                vs = [int(v) for i, v in enumerate(kv.split(' ')) if ((i+1) % 2) == 0]
                kv = [(k, v) for k, v in zip(ks, vs)]
            else:
                kv = []
            if last_id is not None:
                ID += last_id
            if last_lat is not None:
                lat += last_lat
            if last_lon is not None:
                lon += last_lon
            last_id = ID
            last_lat = lat
            last_lon = lon
            yield (ID, lat, lon, kv)





from parosm.types.osmtype import OSMBaseType


class Way(OSMBaseType):
    def __init__(self, identifier, nodes=None, **kwargs):
        """

        :param identifier: Object ID
        :param nodes: IDs of included nodes
        :param user: user name
        :param uid: user id
        :param timestamp: timestamp of last change
        :param visible: object visible?
        :param version: version of the object
        :param changeset: changeset id
        :param tags: dictionary with osm object tags
        """
        super().__init__(identifier, **kwargs)
        self._nodes = list() if nodes is None else nodes

    def add_node(self, identifier):
        """
        Append node to way
        :param identifier: node id
        """
        self._nodes.append(identifier)

    @property
    def nodes(self):
        for node in self._nodes:
            yield node

    def __str__(self):
        attr_str = list()
        attr_str.append('(id={})'.format(self._id))
        attr_str.append('(nodes={})'.format(len(self._nodes)))
        if len(self._tags) > 0:
            attr_str.append('(tags={})'.format(self._tags))
        attr_str = ', '.join(attr_str)
        return 'Way<{}>;'.format(attr_str)



"""
Includes the basic OpenStreetMap types
"""


class OSMBaseType:
    """
    OSM Base type bundles attributes that are shared
    by all OSM types

    :param identifier: object id
    :param user: user name
    :param uid: user id
    :param timestamp: timestamp of last change
    :param visible: object visible?
    :param version: version of the object
    :param changeset: changeset id
    :param tags: dictionary with osm object tags
    """
    def __init__(self,
                 identifier,
                 user=None,
                 uid=None,
                 timestamp=None,
                 visible=None,
                 version=0,
                 changeset=0,
                 tags=None, **kwargs):
        self._id = identifier
        self._user = "" if user is None else user
        self._uid = 0 if uid is None else uid
        self._timestamp = "" if timestamp is None else timestamp
        self._visible = "false" if visible is None else visible
        self._version = 0 if version is None else version
        self._changeset = 0 if changeset is None else changeset
        self._tags = dict() if tags is None else tags

    def add_tag(self, key, value):
        """
        add tag to object
        :param key: key name
        :param value:
        """
        self._tags[key] = value

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, identifier):
        self._id = identifier

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, user):
        self._user = user

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, uid):
        self._uid = uid

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, ts):
        self._timestamp = ts

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, v):
        if not isinstance(v, str):
            raise TypeError('argument not a string')
        if v.lower() == 'false':
            self._visible = 'false'
        elif v.lower() == 'true':
            self._visible = 'true'
        else:
            raise ValueError('Visible is either "true" or "false"')

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, v):
        self._version = v

    @property
    def changeset(self):
        return self._changeset

    @changeset.setter
    def changeset(self, cs):
        self._changeset = cs

    @property
    def tags(self):
        return self._tags

    @tags.setter
    def tags(self, tags):
        if not isinstance(tags, dict):
            raise TypeError('tags should be type <dict>')
        for key, value in tags:
            self._tags[key] = value


class OSM:
    """
    Basic OSM Element including the bounding box
    """
    def __init__(self, version, bbox=None):
        """
        Initialize bounding box
        :param version: Version of open street map
        :param bbox: dict[minlat, minlon, maxlat, maxlon]
        """
        self._version = version
        self._bounds = {
            'minlat': None,
            'minlon': None,
            'maxlat': None,
            'maxlon': None
        }
        if bbox is not None:
            self.set_bounds(**bbox)

    def set_bounds(self,
                   minlat=None,
                   minlon=None,
                   maxlat=None,
                   maxlon=None):
        """
        Set bounding box
        :param minlat: Y-min
        :param minlon: X-min
        :param maxlat: Y-max
        :param maxlon: X-max
        """
        self._bounds['minlat'] = minlat
        self._bounds['minlon'] = minlon
        self._bounds['maxlat'] = maxlat
        self._bounds['maxlon'] = maxlon

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, version):
        self._version = version

    @property
    def bounds(self):
        return self._bounds

    @bounds.setter
    def bounds(self, bounds):
        if not isinstance(bounds, dict):
            raise TypeError('argument is not a dict')
        keys = ['minlat', 'maxlat', 'minlon', 'maxlon']
        for key, value in bounds.items():
            if key in keys:
                self._bounds[key] = value
        for key in keys:
            if key not in bounds:
                raise ValueError('{} not specified')

    def __str__(self):
        attr_str = list()
        attr_str.append('(version={})'.format(self._version))
        for key, value in self._bounds.items():
            attr_str.append('({}={})'.format(key, value))
        attr_str = ', '.join(attr_str)

        return 'OSM<{}>'.format(attr_str)



import collections

from parosm.types.osmtype import OSMBaseType


class Node(OSMBaseType):
    """
    OpenStreetMap Node Type
    """
    def __init__(self, identifier, lat, lon, **kwargs):
        """

        :param identifier: ID of object
        :param lat: latitude
        :param lon: longitude
        :param user: user name
        :param uid: user id
        :param timestamp: timestamp of last change
        :param visible: object visible?
        :param version: version of the object
        :param changeset: changeset id
        :param tags: dictionary with osm object tags
        """
        super().__init__(identifier=identifier, **kwargs)
        self._lat = lat
        self._lon = lon
        self.lat = lat
        self.lon = lon

    @property
    def lat(self):
        return self._lat

    @lat.setter
    def lat(self, val):
        self._lat = float(val)

    @property
    def lon(self):
        return self._lon

    @lon.setter
    def lon(self, val):
        self._lon = float(val)

    @property
    def coords(self):
        return self._lat, self._lon

    @coords.setter
    def coords(self, coords):
        if not isinstance(coords, (collections.Iterable,
                                   collections.Sized,
                                   collections.Mapping)):
            raise TypeError('coords is not iterable')
        if len(coords) != 2:
            raise ValueError('coords does not have length 2')
        self.lat = coords[0]
        self.lon = coords[1]

    def __str__(self):
        attr_str = list()
        attr_str.append('(id={})'.format(self._id))
        attr_str.append('(lat={}, lon={})'.format(self._lat,
                                                  self._lon))
        if len(self._tags) > 0:
            attr_str.append('(tags={})'.format(self._tags))
        attr_str = ', '.join(attr_str)
        return 'Node<{}>;'.format(attr_str)



import unittest
from setuptools import setup, find_packages


def spin_up_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests')
    return test_suite


if __name__ == '__main__':
    with open('README.rst', 'r') as f:
        long_description = f.read()

    setup(
        name='parosm',
        version='0.2',
        description='OpenStreetMap Parser',
        long_description=long_description,
        url='https://github.com/DaGuich/parosm.git',
        keywords=[
            'parser',
            'osm',
            'openstreetmap'
        ],
        license='GPL Version 3',
        classifiers=[
            'Programming Language :: Python :: 3'
        ],
        packages=find_packages(exclude=['tests']),
        install_requires=[
            'protobuf',
            'defusedxml==0.5.0',
            'python-magic==0.4.15'
        ],
        entry_points={
            'console_scripts': [
                'osm2sql = parosm.prog.osm2sql.__init__:main',
                'osminfo = parosm.prog.osminfo.__init__:main'
            ]
        },
        test_suite='setup.spin_up_test_suite',
        python_requires='>=3.0.0',
    )



"""
OpenStreetMap Types
"""
from parosm.types.node import Node
from parosm.types.way import Way
from parosm.types.relation import Relation
from parosm.types.osmtype import OSM

__all__ = ('Node', 'Way', 'Relation', 'OSM')



import argparse


def parse_args():
    parser = argparse.ArgumentParser(
        description='Read a OSM-XML file and put out a SQL Query'
    )
    parser.add_argument('input',
                        type=str,
                        help='OSM file to read')
    parser.add_argument('output',
                        type=str,
                        help='Output path or "-" for stdout')
    return parser.parse_args()


def main():
    args = parse_args()
    print(args.input)
    print(args.output)


if __name__ == '__main__':
    main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fileformat.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='fileformat.proto',
  package='OSMPBF',
  syntax='proto2',
  serialized_pb=_b('\n\x10\x66ileformat.proto\x12\x06OSMPBF\"l\n\x04\x42lob\x12\x0b\n\x03raw\x18\x01 \x01(\x0c\x12\x10\n\x08raw_size\x18\x02 \x01(\x05\x12\x11\n\tzlib_data\x18\x03 \x01(\x0c\x12\x11\n\tlzma_data\x18\x04 \x01(\x0c\x12\x1f\n\x13OBSOLETE_bzip2_data\x18\x05 \x01(\x0c\x42\x02\x18\x01\"?\n\nBlobHeader\x12\x0c\n\x04type\x18\x01 \x02(\t\x12\x11\n\tindexdata\x18\x02 \x01(\x0c\x12\x10\n\x08\x64\x61tasize\x18\x03 \x02(\x05\x42\x11\n\rcrosby.binaryH\x03')
)




_BLOB = _descriptor.Descriptor(
  name='Blob',
  full_name='OSMPBF.Blob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw', full_name='OSMPBF.Blob.raw', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_size', full_name='OSMPBF.Blob.raw_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='zlib_data', full_name='OSMPBF.Blob.zlib_data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lzma_data', full_name='OSMPBF.Blob.lzma_data', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='OBSOLETE_bzip2_data', full_name='OSMPBF.Blob.OBSOLETE_bzip2_data', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=136,
)


_BLOBHEADER = _descriptor.Descriptor(
  name='BlobHeader',
  full_name='OSMPBF.BlobHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='OSMPBF.BlobHeader.type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indexdata', full_name='OSMPBF.BlobHeader.indexdata', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='datasize', full_name='OSMPBF.BlobHeader.datasize', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=138,
  serialized_end=201,
)

DESCRIPTOR.message_types_by_name['Blob'] = _BLOB
DESCRIPTOR.message_types_by_name['BlobHeader'] = _BLOBHEADER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Blob = _reflection.GeneratedProtocolMessageType('Blob', (_message.Message,), dict(
  DESCRIPTOR = _BLOB,
  __module__ = 'fileformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.Blob)
  ))
_sym_db.RegisterMessage(Blob)

BlobHeader = _reflection.GeneratedProtocolMessageType('BlobHeader', (_message.Message,), dict(
  DESCRIPTOR = _BLOBHEADER,
  __module__ = 'fileformat_pb2'
  # @@protoc_insertion_point(class_scope:OSMPBF.BlobHeader)
  ))
_sym_db.RegisterMessage(BlobHeader)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\rcrosby.binaryH\003'))
_BLOB.fields_by_name['OBSOLETE_bzip2_data'].has_options = True
_BLOB.fields_by_name['OBSOLETE_bzip2_data']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)



import os.path
import magic

from parosm.parse.parsebase import BaseParser
from parosm.parse.parsepbf import PBFParser
from parosm.parse.parsexml import XMLParser


class MultiParser(BaseParser):
    def __init__(self, file, callback=None):
        super().__init__(file, callback)
        self.__file = file
        if not os.path.isfile(file):
            raise Exception('is not a file')

        self.__callback = self.__default_callback \
            if callback is None else callback

        file_type = magic.from_file(file)
        if file_type == 'OpenStreetMap XML data':
            self.__parser = XMLParser(self.__file,
                                      self.__callback)
        elif file_type == 'OpenStreetMap Protocolbuffer Binary Format':
            self.__parser = PBFParser(self.__file,
                                      self.__callback)
        else:
            raise Exception('Not a osm file')

    @staticmethod
    def __default_callback(element):
        print(str(element))

    def parse(self):
        self.__parser.parse()
